/**
 * Created by mac on 16/9/8.
 */

const crypto = require('crypto');
const fs = require('fs');
const path = require('path');

let privatePem = null;
let publicPem = null;
const encoding = "hex"; //'base64'

// var serverModulusBit = 1024;
// var serverMaxBit = serverModulusBit / 8;
// var serverRealBit = serverMaxBit - 11;

const constants = require('constants'); //node crypto module

const cryptokit = {

    MAX_SERVER_ENCRYPT_BLOCK: 117,

    //加密
    _cipher: function (algorithm, key, data, iv) {
        let encrypted = "";
        let cip = iv ? crypto.createCipheriv(algorithm, key, iv) : crypto.createCipher(algorithm, key);
        encrypted += cip.update(data, 'utf8', 'base64');
        encrypted += cip.final('base64');
        return encrypted;
    },

    //解密
    _decipher: function (algorithm, key, data, iv) {
        let decrypted = "";
        let decip = iv ? crypto.createDecipheriv(algorithm, key, iv) : crypto.createDecipher(algorithm, key);
        decrypted += decip.update(data);
        decrypted += decip.final();
        return decrypted;
    },

    /**
     * 解密字符串密文
     * @param data  the plaintext to be descrypted in utf8 format
     * @returns {*} the encrypted Plaintext data
     */
    aesDecode: function (key, data, iv) {
        return this._decipher("aes-256-cbc", key, data, iv);
    },

    /**
     * 加密字符串密文
     * @param data  the Plaintext data to be encrypted in utf8 format
     * @returns {*} the decrypted plaintext data
     */
    aesEncode: function (key, data, iv) {
        return this._cipher("aes-256-cbc", key, data, iv);
    },

    /**
     * 解密字符串密文
     * @param data  the plaintext to be descrypted in utf8 format
     * @returns {*} the encrypted Plaintext data
     */
    aes128Decode: function (key, data, iv) {
        return this._decipher("aes-128-cbc", key, data, iv);
    },

    /**
     * 加密字符串密文
     * @param data  the Plaintext data to be encrypted in utf8 format
     * @returns {*} the decrypted plaintext data
     */
    aes128Encode: function (key, data, iv) {
        return this._cipher("aes-128-cbc", key, data, iv);
    },

    _hash: function (algorithm, data) {
        let shasum = crypto.createHash(algorithm);
        shasum.update(data);
        return shasum.digest(encoding);
    },

    /**
     * 计算data的 SHA256 哈希值
     * @param data String to hash
     * @returns {*}
     */
    hashSHA256: function (data) {
        return this._hash("sha256", data);
    },

    /**
     * 计算data的 SHA1 哈希值
     * @param data String to hash
     * @returns {*}
     */
    hashSHA1: function (data) {
        return this._hash("sha1", data);
    },

    /**
     * 签名算法
     * openssl genrsa  -out server.pem 1024
     * openssl req -key server.pem -new -x509 -out cert.pem
     */
    signer: function (algorithm, key, data) {
        let sign = crypto.createSign(algorithm);
        sign.update(data);
        let sig = sign.sign(key, encoding);
        return sig;
    },

    /**
     * 验证签名算法
     * @param algorithm
     * @param pub
     * @param sig
     * @param data
     * @returns {*}
     */
    verify: function (algorithm, pub, sig, data) {
        let verify = crypto.createVerify(algorithm);
        verify.update(data);
        return verify.verify(pub, sig, encoding);
    },

    /**
     * 非对称加密算法RSA加密
     * @param data String
     * @returns {*}
     */
    rsaEncode: function (data) {
        if (!privatePem) {
            privatePem = fs.readFileSync(path.join(__dirname, './private.pem'));
        }
        let privateKey = {key: privatePem.toString(), padding: constants.RSA_PKCS1_PADDING};
        let out = crypto.privateEncrypt(privateKey, new Buffer(data));
        return out.toString(encoding);
    },

    /**
     * 非对称加密算法RSA解密
     * @param data
     * @returns {*}
     */
    rsaDecode: function (data) {
        if (!privatePem) {
            privatePem = fs.readFileSync(path.join(__dirname, './private.pem'));
        }

        let privateKey = {key: privatePem.toString(), padding: constants.RSA_PKCS1_PADDING};
        let out = crypto.privateDecrypt(privateKey, new Buffer(data, encoding));
        return out.toString();
    },

    /**
     * HMAC-SHA1 哈希算法
     *
     * @param key
     * @param data
     * @returns {*}
     */
    hmacSha1Encode: function (key, data) {
        const hmac = crypto.createHmac('sha1', key);
        hmac.update(data);
        return hmac.digest('base64');
    },

    /**
     * 非对称加密算法RSA公钥加密,
     * @param data 缺省用于测试验证
     * @returns {*}
     */
    rsaClientEncode: function (data, pub, encoding) {
        if (!publicPem) {
            let pubKey = pub ? pub : '../public/public.pem';
            publicPem = fs.readFileSync(path.join(__dirname, pubKey));
        }

        let originBuff = new Buffer(data);
        let size = originBuff.length;
        let offset = 0;
        let pubKey = {key: publicPem.toString(), padding: constants.RSA_PKCS1_PADDING};

        let out = '';
        while (offset < size) {
            let originTmp = originBuff.slice(offset, offset + 64 < size ? 64 : size - offset);
            let outTmp = crypto.publicEncrypt(pubKey, originTmp);
            offset += 64;
            out += outTmp.toString(encoding);
        }
        return out;
    },

    /**
     * 非对称加密算法RSA公钥解密
     * @param data 缺省用于测试验证
     * @returns {*}
     */
    rsaClientDecode: function (data, pub, encoding) {
        if (!publicPem) {
            let pubKey = pub ? pub : '../public/public.pem';
            publicPem = fs.readFileSync(path.join(__dirname, pubKey));
        }

        let originBuff = new Buffer(data);
        let size = originBuff.length;
        let offset = 0;
        let pubKey = {key: publicPem.toString(), padding: constants.RSA_PKCS1_PADDING};

        let out = '';
        while (offset < size) {
            let originTmp = originBuff.slice(offset, offset + 64 < size ? 64 : size - offset);
            let outTmp = crypto.publicDecrypt(pubKey, originTmp);
            offset += 64;
            out += outTmp.toString(encoding);
        }
        return out;
    },

};

module.exports = cryptokit;
