/**
 * Created by guxin on 2017/5/12.
 */

"use strict";

const config = require('config'),
    ipAddr = require('ipaddr.js'),
    {log} = require('./logger');

const IpSegRegExp = /^(\d{1,2}|1\d\d|2[0-4]\d|25[0-5])\.(\d{1,2}|1\d\d|2[0-4]\d|25[0-5])\.(\d{1,2}|1\d\d|2[0-4]\d|25[0-5])\.(\d{1,2}|1\d\d|2[0-4]\d|25[0-5])\/([0-9]|[1-2][0-9]|3[0-2])$/;
let filterList = null; //IP过滤列表缓存

module.exports = {

    /**
     * 正则校验是否合法IP网段
     * @param ipSeg
     * @returns {boolean}
     */
    _validateIPSeg: function (ipSeg) {
        return IpSegRegExp.test(ipSeg);
    },

    _ipToMaskBin: function (ip, decMask) {
        let arr = ip.split('.');
        let str = '';
        for (let i = 0; i < 4; i++) {
            let tmp = parseInt(arr[i]).toString(2);
            if (tmp.length < 8) {
                let size = 8 - tmp.length;
                for (let j = 0; j < size; j++) {
                    tmp = '0' + tmp;
                }
            }
            str += tmp;
        }

        //set mask
        let mask = str.substr(0, decMask);
        for (let i = 0; i < 32 - decMask; i++) {
            mask += '0';
        }
        return mask;
    },

    _checkIp: function (ip, ipMask, mask) {
        let ipBinStr = this._ipToMaskBin(ip, 32);
        let ipSegBinStr = this._ipToMaskBin(ipMask, mask);

        let str1 = ipBinStr.substr(0, mask);
        let str2 = ipSegBinStr.substr(0, mask);
        return str1 === str2;
    },

    isValid: function (ip) {
        let self = this;

        if (!config.ipWhiteList) {
            log.warn('IP white list not defined!');
            return true;
        }

        if (!filterList) {
            log.info('creating filterList...');
            filterList = this.filterList(config.ipWhiteList);
        }

        for (let i = 0; i < filterList.length; i++) {
            if (self._checkIp(ip, filterList[i].ipMask, filterList[i].mask)) {
                return true;
            }
        }

        return false;
    },

    validProxy: function (req) {
        if (config.proxy) { //需要验证代理服务器地址
            let size = req.ips.length;
            if (!size) return true;
            let lastIp = req.ips[size - 1];
            let proxyIp = ipAddr.process(lastIp).toString();
            return this.isValid(proxyIp)
        } else {
            return true
        }
    },

    filter: function (req, res, next) {
        let srcIp = ipAddr.process(req.ip).toString();
        // log.debug("srcIp: " + srcIp)
        if (this.isValid(srcIp) && this.validProxy(req)) next();
        else {
            log.warn('access denied: ' + req.ip + ' - ' + req.originalUrl);
            let err = new Error('access denied');
            err.status = 403;
            next(err);
        }
    },

    match: function (req, filterList) {
        let ip = ipAddr.process(req.ip).toString();

        for (let i = 0; i < filterList.length; i++) {
            if (this._checkIp(ip, filterList[i].ipMask, filterList[i].mask)) {
                return true;
            }
        }
        return false;
    },

    filterList: function (ipFilterList) {
        let filterList = [];
        let self = this
        ipFilterList.forEach(function (ipSeg) {
            if (!self._validateIPSeg(ipSeg)) {
                log.error('illegal IP segment definition: ' + ipSeg);
                return;
            }
            let index = ipSeg.indexOf('/');
            let mask = index < 0 ? 0 : parseInt(ipSeg.substr(index + 1));
            let ipMask = index < 0 ? ipSeg : ipSeg.substr(0, index);
            filterList.push({mask: mask, ipMask: ipMask});
        });
        return filterList
    }

};
